@@ -1,4 +1,4 @@
-# u-msgpack-python v2.6.0 - v at sergeev.io
+# u-msgpack-python v2.7.0 - v at sergeev.io
 # https://github.com/vsergeev/u-msgpack-python
 #
 # u-msgpack-python is a lightweight MessagePack serializer and deserializer
@@ -31,7 +31,7 @@
 # THE SOFTWARE.
 #
 """
-u-msgpack-python v2.6.0 - v at sergeev.io
+u-msgpack-python v2.7.0 - v at sergeev.io
 https://github.com/vsergeev/u-msgpack-python
 
 u-msgpack-python is a lightweight MessagePack serializer and deserializer
@@ -54,10 +54,10 @@ if sys.version_info[0:2] >= (3, 3):
 else:
     from collections import Hashable
 
-__version__ = "2.6.0"
+__version__ = "2.7.0"
 "Module version string"
 
-version = (2, 6, 0)
+version = (2, 7, 0)
 "Module version tuple"
 
 
@@ -140,7 +140,8 @@ class InvalidString(bytes):
 # Ext Serializable Decorator
 ##############################################################################
 
-_ext_classes = {}
+_ext_class_to_type = {}
+_ext_type_to_class = {}
 
 
 def ext_serializable(ext_type):
@@ -159,13 +160,13 @@ def ext_serializable(ext_type):
             Ext type or class already registered.
     """
     def wrapper(cls):
-        if ext_type in _ext_classes:
-            raise ValueError("Ext type 0x{:02x} already registered with class {:s}".format(ext_type, repr(_ext_classes[ext_type])))
-        elif cls in _ext_classes:
+        if ext_type in _ext_type_to_class:
+            raise ValueError("Ext type 0x{:02x} already registered with class {:s}".format(ext_type, repr(_ext_type_to_class[ext_type])))
+        elif cls in _ext_class_to_type:
             raise ValueError("Class {:s} already registered with Ext type 0x{:02x}".format(repr(cls), ext_type))
 
-        _ext_classes[ext_type] = cls
-        _ext_classes[cls] = ext_type
+        _ext_type_to_class[ext_type] = cls
+        _ext_class_to_type[cls] = ext_type
 
         return cls
 
@@ -472,9 +473,9 @@ def _pack2(obj, fp, **options):
         _pack_nil(obj, fp, options)
     elif ext_handlers and obj.__class__ in ext_handlers:
         _pack_ext(ext_handlers[obj.__class__](obj), fp, options)
-    elif obj.__class__ in _ext_classes:
+    elif obj.__class__ in _ext_class_to_type:
         try:
-            _pack_ext(Ext(_ext_classes[obj.__class__], obj.packb()), fp, options)
+            _pack_ext(Ext(_ext_class_to_type[obj.__class__], obj.packb()), fp, options)
         except AttributeError:
             raise NotImplementedError("Ext serializable class {:s} is missing implementation of packb()".format(repr(obj.__class__)))
     elif isinstance(obj, bool):
@@ -507,6 +508,16 @@ def _pack2(obj, fp, **options):
         else:
             raise UnsupportedTypeException(
                 "unsupported type: %s" % str(type(obj)))
+    elif _ext_class_to_type:
+        # Linear search for superclass
+        t = next((t for t in _ext_class_to_type if isinstance(obj, t)), None)
+        if t:
+            try:
+                _pack_ext(Ext(_ext_class_to_type[t], obj.packb()), fp, options)
+            except AttributeError:
+                raise NotImplementedError("Ext serializable class {:s} is missing implementation of packb()".format(repr(t)))
+        else:
+            raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
     else:
         raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
 
@@ -549,9 +560,9 @@ def _pack3(obj, fp, **options):
         _pack_nil(obj, fp, options)
     elif ext_handlers and obj.__class__ in ext_handlers:
         _pack_ext(ext_handlers[obj.__class__](obj), fp, options)
-    elif obj.__class__ in _ext_classes:
+    elif obj.__class__ in _ext_class_to_type:
         try:
-            _pack_ext(Ext(_ext_classes[obj.__class__], obj.packb()), fp, options)
+            _pack_ext(Ext(_ext_class_to_type[obj.__class__], obj.packb()), fp, options)
         except AttributeError:
             raise NotImplementedError("Ext serializable class {:s} is missing implementation of packb()".format(repr(obj.__class__)))
     elif isinstance(obj, bool):
@@ -584,6 +595,16 @@ def _pack3(obj, fp, **options):
         else:
             raise UnsupportedTypeException(
                 "unsupported type: %s" % str(type(obj)))
+    elif _ext_class_to_type:
+        # Linear search for superclass
+        t = next((t for t in _ext_class_to_type if isinstance(obj, t)), None)
+        if t:
+            try:
+                _pack_ext(Ext(_ext_class_to_type[t], obj.packb()), fp, options)
+            except AttributeError:
+                raise NotImplementedError("Ext serializable class {:s} is missing implementation of packb()".format(repr(t)))
+        else:
+            raise UnsupportedTypeException("unsupported type: %s" % str(type(obj)))
     else:
         raise UnsupportedTypeException(
             "unsupported type: %s" % str(type(obj)))
@@ -799,11 +820,11 @@ def _unpack_ext(code, fp, options):
         return ext_handlers[ext_type](Ext(ext_type, ext_data))
 
     # Unpack with ext classes, if type is registered
-    if ext_type in _ext_classes:
+    if ext_type in _ext_type_to_class:
         try:
-            return _ext_classes[ext_type].unpackb(ext_data)
+            return _ext_type_to_class[ext_type].unpackb(ext_data)
         except AttributeError:
-            raise NotImplementedError("Ext serializable class {:s} is missing implementation of unpackb()".format(repr(_ext_classes[ext_type])))
+            raise NotImplementedError("Ext serializable class {:s} is missing implementation of unpackb()".format(repr(_ext_type_to_class[ext_type])))
 
     # Timestamp extension
     if ext_type == -1:
